@@ -109,6 +109,7 @@ gst_spider_identity_get_type (void)
       0,
       (GInstanceInitFunc) gst_spider_identity_init,
     };
+
     spider_identity_type =
         g_type_register_static (GST_TYPE_ELEMENT, "GstSpiderIdentity",
         &spider_identity_info, 0);
@@ -172,16 +173,16 @@ gst_spider_identity_chain (GstPad * pad, GstBuffer * buf)
 {
   GstSpiderIdentity *ident;
 
-  /*g_print ("chaining on pad %s:%s with buffer %p\n", GST_DEBUG_PAD_NAME (pad), buf); */
-
   g_return_if_fail (pad != NULL);
   g_return_if_fail (GST_IS_PAD (pad));
+
   if (buf == NULL)
     return;
 
   ident = GST_SPIDER_IDENTITY (gst_pad_get_parent (pad));
 
   if (GST_IS_EVENT (buf)) {
+    GST_DEBUG_OBJECT (ident, "spider identity received event %p", buf);
     /* start hack for current event stuff here */
     /* check for unlinked elements and send them the EOS event, too */
     if (GST_EVENT_TYPE (GST_EVENT (buf)) == GST_EVENT_EOS) {
@@ -193,23 +194,31 @@ gst_spider_identity_chain (GstPad * pad, GstBuffer * buf)
 
         list = g_list_next (list);
         if (conn->current != (GstElement *) conn->src) {
-          GST_DEBUG ("sending EOS to unconnected element %s from %s",
-              GST_ELEMENT_NAME (conn->src), GST_ELEMENT_NAME (ident));
-          gst_pad_push (conn->src->src,
-              GST_DATA (GST_BUFFER (gst_event_new (GST_EVENT_EOS))));
+          GstEvent *event;
+
+          event = gst_event_new (GST_EVENT_EOS);
+          GST_DEBUG_OBJECT (ident,
+              "sending EOS event %p to unconnected element %s from %s",
+              event, GST_ELEMENT_NAME (conn->src), GST_ELEMENT_NAME (ident));
+          gst_pad_push (conn->src->src, GST_DATA (event));
           gst_element_set_eos (GST_ELEMENT (conn->src));
         }
       }
     }
     /* end hack for current event stuff here */
+    GST_DEBUG_OBJECT (ident,
+        "calling default handler for event %p on pad %s:%s",
+        buf, GST_DEBUG_PAD_NAME (pad));
 
     gst_pad_event_default (pad, GST_EVENT (buf));
     return;
   }
 
   if ((ident->src != NULL) && (GST_PAD_PEER (ident->src) != NULL)) {
-    /* g_print("pushing buffer %p (refcount %d - buffersize %d) to pad %s:%s\n", buf, GST_BUFFER_REFCOUNT (buf), GST_BUFFER_SIZE (buf), GST_DEBUG_PAD_NAME (ident->src)); */
-    GST_LOG ("push %p %" G_GINT64_FORMAT, buf, GST_BUFFER_OFFSET (buf));
+    GST_LOG_OBJECT (ident, "pushing buffer %p "
+        "(refcount %d, size %u, offset %" G_GINT64_FORMAT ") ",
+        buf, GST_BUFFER_REFCOUNT_VALUE (buf),
+        GST_BUFFER_SIZE (buf), GST_BUFFER_OFFSET (buf));
     gst_pad_push (ident->src, GST_DATA (buf));
   } else if (GST_IS_BUFFER (buf)) {
     gst_buffer_unref (buf);
@@ -462,7 +471,8 @@ typedef struct
   GstBuffer *buffer;
   guint best_probability;
   GstCaps *caps;
-} SpiderTypeFind;
+}
+SpiderTypeFind;
 guint8 *
 spider_find_peek (gpointer data, gint64 offset, guint size)
 {
