@@ -71,7 +71,7 @@ enum
   ARG_SILENT,
   ARG_DUMP,
   ARG_PARENTSIZE,
-  ARG_LAST_MESSAGE,
+  ARG_LAST_MESSAGE
 };
 
 GstStaticPadTemplate fakesrc_src_template = GST_STATIC_PAD_TEMPLATE ("src%d",
@@ -238,7 +238,7 @@ gst_fakesrc_class_init (GstFakeSrcClass * klass)
           G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_NUM_BUFFERS,
       g_param_spec_int ("num-buffers", "num-buffers",
-          "Number of buffers to output before sending EOS", G_MININT, G_MAXINT,
+          "Number of buffers to output before sending EOS", -1, G_MAXINT,
           0, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_EOS,
       g_param_spec_boolean ("eos", "eos", "Send out the EOS event?", TRUE,
@@ -535,6 +535,7 @@ gst_fakesrc_set_property (GObject * object, guint prop_id, const GValue * value,
       src->dump = g_value_get_boolean (value);
       break;
     default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
   }
 }
@@ -872,7 +873,6 @@ gst_fakesrc_change_state (GstElement * element)
     case GST_STATE_READY_TO_NULL:
       break;
     default:
-      g_assert_not_reached ();
       break;
   }
 
