@@ -25,6 +25,7 @@
 
 #include <string.h>
 
+#include "gst_private.h"
 #include <gst/gst.h>
 #include <gobject/gvaluecollector.h>
 
@@ -247,6 +248,22 @@ gst_structure_get_name (const GstStructure * structure)
   return g_quark_to_string (structure->name);
 }
 
+/**
+ * gst_structure_get_name:
+ * @structure: a #GstStructure
+ *
+ * Accessor fuction.
+ *
+ * Returns: the quark representing the name of the structure.
+ */
+GQuark
+gst_structure_get_name_id (const GstStructure * structure)
+{
+  g_return_val_if_fail (structure != NULL, 0);
+
+  return structure->name;
+}
+
 /**
  * gst_structure_set_name:
  * @structure: a #GstStructure
@@ -411,6 +428,13 @@ gst_structure_set_valist (GstStructure * structure,
 
           g_value_init (&field.value, GST_TYPE_BUFFER);
           g_value_set_boxed (&field.value, buffer);
+        } else if (type == GST_TYPE_FRACTION) {
+          gint n, d;
+          n = va_arg (varargs, int);
+          d = va_arg (varargs, int);
+
+          g_value_init (&field.value, GST_TYPE_FRACTION);
+          gst_value_set_fraction (&field.value, n, d);
         } else {
           g_critical ("unimplemented vararg field type %d\n", (int) type);
           return;
@@ -967,7 +991,11 @@ static GstStructureAbbreviation gst_structure_abbrs[] = {
   {"f", G_TYPE_FLOAT},
   {"double", G_TYPE_DOUBLE},
   {"d", G_TYPE_DOUBLE},
+/* these are implemented with strcmp below */
+//{ "buffer",   GST_TYPE_BUFFER },
 //{ "fourcc",   GST_TYPE_FOURCC },
+//{ "4",   GST_TYPE_FOURCC },
+//{ "fraction",   GST_TYPE_FRACTION },
   {"boolean", G_TYPE_BOOLEAN},
   {"bool", G_TYPE_BOOLEAN},
   {"b", G_TYPE_BOOLEAN},
@@ -996,6 +1024,9 @@ gst_structure_from_abbr (const char *type_name)
   if (strcmp (type_name, "buffer") == 0) {
     return GST_TYPE_BUFFER;
   }
+  if (strcmp (type_name, "fraction") == 0) {
+    return GST_TYPE_FRACTION;
+  }
 
   return g_type_from_name (type_name);
 }
@@ -1020,10 +1051,35 @@ gst_structure_to_abbr (GType type)
   if (type == GST_TYPE_BUFFER) {
     return "buffer";
   }
+  if (type == GST_TYPE_FRACTION) {
+    return "fraction";
+  }
 
   return g_type_name (type);
 }
 
+static GType
+gst_structure_value_get_generic_type (GValue * val)
+{
+  if (G_VALUE_TYPE (val) == GST_TYPE_LIST
+      || G_VALUE_TYPE (val) == GST_TYPE_FIXED_LIST) {
+    GArray *array = g_value_peek_pointer (val);
+
+    if (array->len > 0) {
+      GValue *value = &g_array_index (array, GValue, 0);
+
+      return gst_structure_value_get_generic_type (value);
+    } else {
+      return G_TYPE_INT;
+    }
+  } else if (G_VALUE_TYPE (val) == GST_TYPE_INT_RANGE) {
+    return G_TYPE_INT;
+  } else if (G_VALUE_TYPE (val) == GST_TYPE_DOUBLE_RANGE) {
+    return G_TYPE_DOUBLE;
+  }
+  return G_VALUE_TYPE (val);
+}
+
 #define GST_ASCII_IS_STRING(c) (g_ascii_isalnum((c)) || ((c) == '_') || \
       ((c) == '-') || ((c) == '+') || ((c) == '/') || ((c) == ':') || \
       ((c) == '.'))
@@ -1061,23 +1117,8 @@ gst_structure_to_string (const GstStructure * structure)
     field = GST_STRUCTURE_FIELD (structure, i);
 
     t = gst_value_serialize (&field->value);
-    type = G_VALUE_TYPE (&field->value);
+    type = gst_structure_value_get_generic_type (&field->value);
 
-    if (type == GST_TYPE_LIST) {
-      GArray *array = g_value_peek_pointer (&field->value);
-
-      if (array->len > 0) {
-        GValue *value = &g_array_index (array, GValue, 0);
-
-        type = G_VALUE_TYPE (value);
-      } else {
-        type = G_TYPE_INT;
-      }
-    } else if (G_VALUE_TYPE (&field->value) == GST_TYPE_INT_RANGE) {
-      type = G_TYPE_INT;
-    } else if (G_VALUE_TYPE (&field->value) == GST_TYPE_DOUBLE_RANGE) {
-      type = G_TYPE_DOUBLE;
-    }
     g_string_append_printf (s, ", %s=(%s)%s", g_quark_to_string (field->name),
         gst_structure_to_abbr (type), t);
     g_free (t);
@@ -1195,22 +1236,23 @@ gst_structure_parse_range (gchar * s, gchar ** after, GValue * value,
 }
 
 static gboolean
-gst_structure_parse_list (gchar * s, gchar ** after, GValue * value, GType type)
+gst_structure_parse_any_list (gchar * s, gchar ** after, GValue * value,
+    GType type, GType list_type, char begin, char end)
 {
   GValue list_value = { 0 };
   gboolean ret;
   GArray *array;
 
-  g_value_init (value, GST_TYPE_LIST);
+  g_value_init (value, list_type);
   array = g_value_peek_pointer (value);
 
-  if (*s != '{')
+  if (*s != begin)
     return FALSE;
   s++;
 
   while (g_ascii_isspace (*s))
     s++;
-  if (*s == '}') {
+  if (*s == end) {
     s++;
     *after = s;
     return TRUE;
@@ -1225,7 +1267,7 @@ gst_structure_parse_list (gchar * s, gchar ** after, GValue * value, GType type)
   while (g_ascii_isspace (*s))
     s++;
 
-  while (*s != '}') {
+  while (*s != end) {
     if (*s != ',')
       return FALSE;
     s++;
@@ -1249,6 +1291,21 @@ gst_structure_parse_list (gchar * s, gchar ** after, GValue * value, GType type)
   return TRUE;
 }
 
+static gboolean
+gst_structure_parse_list (gchar * s, gchar ** after, GValue * value, GType type)
+{
+  return gst_structure_parse_any_list (s, after, value, type, GST_TYPE_LIST,
+      '{', '}');
+}
+
+static gboolean
+gst_structure_parse_fixed_list (gchar * s, gchar ** after, GValue * value,
+    GType type)
+{
+  return gst_structure_parse_any_list (s, after, value, type,
+      GST_TYPE_FIXED_LIST, '<', '>');
+}
+
 static gboolean
 gst_structure_parse_simple_string (gchar * str, gchar ** end)
 {
@@ -1352,6 +1409,8 @@ gst_structure_parse_value (gchar * str,
     ret = gst_structure_parse_range (s, &s, value, type);
   } else if (*s == '{') {
     ret = gst_structure_parse_list (s, &s, value, type);
+  } else if (*s == '<') {
+    ret = gst_structure_parse_fixed_list (s, &s, value, type);
   } else {
     value_s = s;
     if (!gst_structure_parse_string (s, &value_end, &s))
