@@ -27,7 +27,6 @@
 #include "gstmarshal.h"
 #include "gstscheduler.h"
 #include "gstinfo.h"
-#include "gstlog.h"
 
 #define GST_CAT_DEFAULT GST_CAT_THREAD
 #define STACK_SIZE 0x200000
@@ -98,6 +97,7 @@ gst_thread_priority_get_type (void)
     {G_THREAD_PRIORITY_URGENT, "URGENT", "Urgent Scheduling"},
     {0, NULL, NULL},
   };
+
   if (!thread_priority_type) {
     thread_priority_type =
         g_enum_register_static ("GstThreadPriority", thread_priority);
@@ -123,10 +123,11 @@ gst_thread_get_type (void)
       NULL,
       NULL,
       sizeof (GstThread),
-      4,
+      0,
       gst_thread_init,
       NULL
     };
+
     thread_type = g_type_register_static (GST_TYPE_BIN, "GstThread",
         &thread_info, 0);
   }
@@ -222,6 +223,8 @@ gst_thread_dispose (GObject * object)
 
   g_assert (GST_STATE (thread) == GST_STATE_NULL);
 
+  GST_CAT_DEBUG (GST_CAT_REFCOUNTING, "GstThread: dispose, freeing locks");
+
   g_mutex_free (thread->lock);
   g_cond_free (thread->cond);
 
@@ -297,7 +300,7 @@ gst_thread_new (const gchar * name)
 /**
  * gst_thread_get_current:
  *
- * Create a new thread with the given name.
+ * Gets the current GstThread.
  *
  * Returns: The current GstThread or NULL if you are not running inside a 
  *          #GstThread.
@@ -368,12 +371,14 @@ gst_thread_catch (GstThread * thread)
   if (thread == gst_thread_get_current ()) {
     /* we're trying to catch ourself */
     if (!GST_FLAG_IS_SET (thread, GST_THREAD_MUTEX_LOCKED)) {
+      GST_DEBUG_OBJECT (thread, "catching itself, grabbing lock");
       g_mutex_lock (thread->lock);
       GST_FLAG_SET (thread, GST_THREAD_MUTEX_LOCKED);
     }
     GST_DEBUG_OBJECT (thread, "catching itself");
     GST_FLAG_UNSET (thread, GST_THREAD_STATE_SPINNING);
   } else {
+    GST_DEBUG_OBJECT (thread, "catching thread, grabbing lock");
     /* another thread is trying to catch us */
     g_mutex_lock (thread->lock);
     wait = !GST_FLAG_IS_SET (thread, GST_THREAD_STATE_SPINNING);
@@ -500,14 +505,6 @@ gst_thread_change_state (GstElement * element)
       /* it should be dead now */
       break;
     default:
-      GST_ERROR_OBJECT (element, "unhandled state change! %x",
-          GST_STATE_TRANSITION (element));
-      g_warning ("thread %s: UNHANDLED STATE CHANGE! %x",
-          GST_STR_NULL (GST_OBJECT_NAME (element)),
-          GST_STATE_TRANSITION (element));
-      /* FIXME: not doable with current threading mess:
-         g_assert_not_reached ();
-       */
       break;
   }
 
@@ -543,7 +540,7 @@ gst_thread_child_state_change (GstBin * bin, GstElementState oldstate,
       gst_element_state_get_name (newstate));
   if (parent_class->child_state_change)
     parent_class->child_state_change (bin, oldstate, newstate, element);
-  /* We'll wake up the main thread now. Note that we can't lock the thread here, 
+  /* We'll wake up the main thread now. Note that we can't lock the thread here,
      because we might be called from inside gst_thread_change_state when holding
      the lock. But this doesn't cause any problems. */
   if (newstate == GST_STATE_PLAYING)
@@ -562,6 +559,7 @@ gst_thread_main_loop (void *arg)
 {
   GstThread *thread = NULL;
   gboolean status;
+  GstScheduler *sched;
 
   thread = GST_THREAD (arg);
   g_mutex_lock (thread->lock);
@@ -583,6 +581,8 @@ gst_thread_main_loop (void *arg)
       while (status && GST_FLAG_IS_SET (thread, GST_THREAD_STATE_SPINNING)) {
         g_mutex_unlock (thread->lock);
         status = gst_bin_iterate (GST_BIN (thread));
+        if (!status)
+          GST_DEBUG_OBJECT (thread, "iterate returned false");
         if (GST_FLAG_IS_SET (thread, GST_THREAD_MUTEX_LOCKED)) {
           GST_FLAG_UNSET (thread, GST_THREAD_MUTEX_LOCKED);
         } else {
@@ -600,7 +600,9 @@ gst_thread_main_loop (void *arg)
 
   /* we need to destroy the scheduler here because it has mapped it's
    * stack into the threads stack space */
-  gst_scheduler_reset (GST_ELEMENT_SCHED (thread));
+  sched = GST_ELEMENT_SCHED (thread);
+  if (sched)
+    gst_scheduler_reset (sched);
 
   /* must do that before releasing the lock - we might get disposed before being done */
   g_signal_emit (G_OBJECT (thread), gst_thread_signals[SHUTDOWN], 0);
